@@ -62,7 +62,8 @@ setClass(Class = "MxDataStatic",
 	     minVariance = "numeric",
 	   algebra = "MxOptionalCharOrNumber",
     warnNPDacov = "logical",
-    exoFree = "MxOptionalMatrix"))
+    exoFree = "MxOptionalMatrix",
+    naAction = "character"))
 
 setClass(Class = "MxDataDynamic",
 	 contains = "NonNullData",
@@ -76,7 +77,7 @@ setClassUnion("MxData", c("NULL", "MxDataStatic", "MxDataDynamic"))
 setMethod("initialize", "MxDataStatic",
 	  function(.Object, observed, means, type, numObs, observedStats,
 		   sort, primaryKey, weight, frequency, verbose, .parallel, .noExoOptimize,
-		   minVariance, algebra, warnNPDacov, exoFree) {
+		   minVariance, algebra, warnNPDacov, exoFree, naAction) {
 		.Object@observed <- observed
 		.Object@means <- means
 		.Object@type <- type
@@ -105,6 +106,7 @@ setMethod("initialize", "MxDataStatic",
 		.Object@algebra <- algebra
 		.Object@warnNPDacov <- warnNPDacov
 		.Object@exoFree <- exoFree
+		.Object@naAction <- naAction
 		return(.Object)
 	}
 )
@@ -164,7 +166,7 @@ mxData <- function(observed, type, means = NA, numObs = NA, acov=NA, fullWeight=
 		   observedStats=NA, sort=NA, primaryKey = as.character(NA), weight = as.character(NA),
 		   frequency = as.character(NA), verbose=0L, .parallel=TRUE, .noExoOptimize=TRUE,
 		   minVariance=sqrt(.Machine$double.eps), algebra=c(),
-		   warnNPDacov=TRUE, exoFree=NULL) {
+		   warnNPDacov=TRUE, exoFree=NULL, naAction=c("pass","fail","omit","exclude")) {
   prohibitDotdotdot(list(...))
 	if (length(means) == 1 && is.na(means)) means <- as.numeric(NA)
 	if (missing(observed) || !is(observed, "MxDataFrameOrMatrix")) {
@@ -182,6 +184,7 @@ mxData <- function(observed, type, means = NA, numObs = NA, acov=NA, fullWeight=
 			paste(imxDataTypes[1:(numTypes-1)], collapse="' '"),
 			"' or '", imxDataTypes[numTypes], "'", sep=""))
 	}
+  naAction <- match.arg(naAction)
 	if (type == "acov") {
 		return(legacyMxData(observed, type="acov", means=means, numObs=numObs, 
 			acov=acov, fullWeight=fullWeight, thresholds=thresholds))
@@ -290,7 +293,7 @@ mxData <- function(observed, type, means = NA, numObs = NA, acov=NA, fullWeight=
 	return(new("MxDataStatic", observed, means, type, as.numeric(numObs),
 		observedStats, sort, primaryKey, weight, frequency, as.integer(verbose),
 		as.logical(.parallel), as.logical(.noExoOptimize), minVariance,
-		as.character(algebra), as.logical(warnNPDacov), exoFree))
+		as.character(algebra), as.logical(warnNPDacov), exoFree, naAction))
 }
 
 setGeneric("preprocessDataForBackend", # DEPRECATED
@@ -331,11 +334,6 @@ setMethod("qualifyNames", signature("MxDataStatic"),
 
 setMethod("convertDataForBackend", signature("MxDataStatic"),
 	  function(data, model, flatModel) {
-		  if (data@type == "cor") {
-			  warning(paste("OpenMx does not yet correctly handle mxData(type='cor')",
-					'standard errors and fit statistics.',
-					'See Steiger (1980), "Tests for comparing elements of a correlation matrix".'))
-		  }
 		  if (data@type == "raw") {
 			  if (is.matrix(data@observed) && is.integer(data@observed)) {
 				  data@observed <- matrix(as.double(data@observed),
