@@ -14,7 +14,7 @@
 #   limitations under the License.
 
 mxEval <- function(expression, model, compute = FALSE, show = FALSE, defvar.row = 1L,
-			cache = new.env(parent = emptyenv()), cacheBack = FALSE) {
+			cache = new.env(parent = emptyenv()), cacheBack = FALSE, .extraBack=0L) {
 	if (missing(expression)) {
 		stop("'expression' argument is mandatory in call to mxEval function")
 	} else if (missing(model)) {
@@ -22,8 +22,9 @@ mxEval <- function(expression, model, compute = FALSE, show = FALSE, defvar.row
 	}
 	expression <- match.call()$expression
 	modelvariable <- match.call()$model
+  #for (x in 1:sys.nframe()) cat(x-1, head(ls(parent.frame(x))), fill=TRUE)
 	EvalInternal(expression, model, modelvariable, compute, show, defvar.row,
-		       cache, cacheBack, 2L)
+		       cache, cacheBack, 2L + .extraBack)
 }
 
 EvalInternal <- function(expression, model, modelvariable, compute, show, defvar.row,
@@ -56,7 +57,13 @@ EvalInternal <- function(expression, model, modelvariable, compute, show, defvar
 	if (!is.vector(result)) {
 		result <- eval(result, envir = env)
 	}
-	if (!is.matrix(result)) result <- as.matrix(result)
+	if (!is.matrix(result)) {
+    if (is(result, "MxAlgebra")) {
+      stop(paste("You have referenced algebra",
+                 omxQuotes(result$name), "outside of the model"))
+    }
+    result <- as.matrix(result)
+  }
 	if (cacheBack) {
 		return(list(result, cache))
 	} else {
@@ -187,7 +194,13 @@ evaluateSymbol <- function(symbol, contextString, model, labelsData,
 				result <- definitionStartingValue(key, contextString, model, defvar.row)
 			} else if (is.null(lookup)) {
 				if (!show && !outsideAlgebra && exists(key, envir = env)) {
-					result <- as.matrix(get(key, envir = env))
+					result <- try(as.matrix(get(key, envir = env)), silent=TRUE)
+          if (is(result, 'try-error')){
+          	stop(paste(
+          		"cannot coerce object with symbol", omxQuotes(key), "to a matrix\n",
+          		"(hint: did you neglect to put an object named", omxQuotes(key), "into model",
+          		omxQuotes(model@name),"?)"))
+          }
 				} else {
 					result <- symbol
 				}
@@ -410,14 +423,13 @@ generateLabelsMatrix <- function(modelName, matrix, labelsData) {
 }
 
 mxEvalByName <- function(name, model, compute=FALSE, show=FALSE, defvar.row = 1L,
-		cache = new.env(parent = emptyenv()), cacheBack = FALSE) {
+		cache = new.env(parent = emptyenv()), cacheBack = FALSE, .extraBack=0L) {
    if((length(name) != 1) || typeof(name) != "character") {
       stop("'name' argument must be a character argument")
    }
-   if(!is(model, "MxModel")) {
-      stop("'model' argument must be a MxModel object")
-   }
-   eval(substitute(mxEval(x, model, compute, show, defvar.row, cache, cacheBack),
+   warnModelCreatedByOldVersion(model)
+#for (x in 1:sys.nframe()) cat(x-1, head(ls(parent.frame(x))), fill=TRUE)
+   eval(substitute(mxEval(x, model, compute, show, defvar.row, cache, cacheBack, 1L + .extraBack),
       list(x = as.symbol(name))))
 }
 
@@ -450,6 +462,7 @@ omxBootstrapEvalCov <- function(expression, model, defvar.row = 1L, ...) {
 
 mxBootstrapEval <- function(expression, model, defvar.row = 1L, ..., bq=c(.25,.75),
 				method=c('bcbci','quantile')) {
+  warnModelCreatedByOldVersion(model)
   expression <- match.call()$expression
   modelvariable <- match.call()$model
 	method <- match.arg(method)
@@ -463,6 +476,7 @@ mxBootstrapEvalByName <- function(name, model, defvar.row=1L, ..., bq=c(.25,.75)
    if((length(name) != 1) || typeof(name) != "character") {
       stop("'name' argument must be a character argument")
    }
+  warnModelCreatedByOldVersion(model)
    method <- match.arg(method)
    eval(substitute(mxBootstrapEval(x, model, defvar.row, bq=bq, method=method),
       list(x = as.symbol(name))))
