@@ -209,6 +209,16 @@ static void ensureElemConform(const char *opName, FitContext *fc, omxMatrix **ma
 
 	if (mat0->cols == mat1->cols && mat0->rows == mat1->rows) {
 		omxResizeMatrix(result, mat0->rows, mat0->cols);
+		// copy dimnames if it is unambiguous what to copy
+		if (!result->hasDimnames()) {
+			if (mat0->hasDimnames() && (!mat1->hasDimnames() || mat0->sameDimnames(mat1))) {
+				result->rownames = mat0->rownames;
+				result->colnames = mat0->colnames;
+			} else if (mat1->hasDimnames() && !mat0->hasDimnames()) {
+				result->rownames = mat1->rownames;
+				result->colnames = mat1->colnames;
+			}
+		}
 		return;
 	}
 
@@ -1870,13 +1880,7 @@ static void omxMatrixVech(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		}
 	}
 
-	if(counter != size) {
-		char *errstr = (char*) calloc(250, sizeof(char));
-		sprintf(errstr, "Internal mxThrow in vech().\n");
-		omxRaiseError(errstr);
-		free(errstr);
-	}
-
+	if(counter != size) omxRaiseErrorf("vech: %d != %d", counter, size);
 }
 
 static void omxMatrixVechs(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -1903,13 +1907,7 @@ static void omxMatrixVechs(FitContext *fc, omxMatrix** matList, int numArgs, omx
 		}
 	}
 
-	if(counter != size) {
-		char *errstr = (char*) calloc(250, sizeof(char));
-		sprintf(errstr, "Internal mxThrow in vechs().\n");
-		omxRaiseError(errstr);
-		free(errstr);
-	}
-
+	if(counter != size) mxThrow("Internal error in vechs()");
 }
 
 static void omxRowVectorize(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -1959,7 +1957,7 @@ static void omxColVectorize(FitContext *fc, omxMatrix** matList, int numArgs, om
 static void omxSequenceGenerator(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
 {
 	double start = omxVectorElement(matList[0], 0);
-	double stop = omxVectorElement(matList[1], 0);
+	double stop1 = omxVectorElement(matList[1], 0);
 
 	if (!R_finite(start)) {
 		char *errstr = (char*) calloc(250, sizeof(char));
@@ -1969,7 +1967,7 @@ static void omxSequenceGenerator(FitContext *fc, omxMatrix** matList, int numArg
 		return;
 	}
 
-	if (!R_finite(stop)) {
+	if (!R_finite(stop1)) {
 		char *errstr = (char*) calloc(250, sizeof(char));
 		sprintf(errstr, "Non-finite stop value in ':' operator.\n");
 		omxRaiseError(errstr);
@@ -1977,7 +1975,7 @@ static void omxSequenceGenerator(FitContext *fc, omxMatrix** matList, int numArg
 		return;
 	}
 
-	double difference = stop - start;
+	double difference = stop1 - start;
 	if (difference < 0) difference = - difference;
 
 	int size = ((int) difference) + 1;
@@ -1993,14 +1991,14 @@ static void omxSequenceGenerator(FitContext *fc, omxMatrix** matList, int numArg
 	}
 
 	int count = 0;
-	if ((stop - start) >= 0) {
-		while (start <= stop) {
+	if ((stop1 - start) >= 0) {
+		while (start <= stop1) {
 			omxSetVectorElement(result, count, start);
 			start = start + 1.0;
 			count++;
 		}
 	} else {
-		while (start >= stop) {
+		while (start >= stop1) {
 			omxSetVectorElement(result, count, start);
 			start = start - 1.0;
 			count++;
@@ -2704,24 +2702,38 @@ template <typename T>
 void buildFilterVec(omxMatrix *origCov, omxMatrix *newCov, std::vector<T> &filter)
 {
 	if (origCov->rows != origCov->cols) mxThrow("'%s' must be square", origCov->name());
-	if (origCov->rows != int(origCov->rownames.size())) mxThrow("'%s' must have dimnames", origCov->name());
 	if (newCov->rows != newCov->cols) mxThrow("'%s' must be square", newCov->name());
-	if (newCov->rows != int(newCov->rownames.size())) mxThrow("'%s' must have dimnames", newCov->name());
-	for (int r1=0; r1 < int(newCov->rownames.size()); ++r1) {
-		bool found = false;
-		for (int r2=0; r2 < int(origCov->rownames.size()); ++r2) {
-			if (strEQ(newCov->rownames[r1], origCov->rownames[r2])) {
-				if (filter[r2]) {
-					omxRaiseErrorf("Cannot filter row '%s' in '%s' more than once",
-						       newCov->rownames[r1], origCov->name());
+
+	if (newCov->sameSize(origCov)) {
+		EigenMatrixAdaptor EorigCov(origCov);
+		EigenMatrixAdaptor EnewCov(newCov);
+		Eigen::Array<bool, Eigen::Dynamic, Eigen::Dynamic> NotMatch =
+			EorigCov.array() != EnewCov.array();
+		Eigen::Array<bool, Eigen::Dynamic, 1> mask =
+			NotMatch.rowwise().maxCoeff() || NotMatch.colwise().maxCoeff().transpose();
+		std::copy(&mask.coeffRef(0), &mask.coeffRef(mask.size()), filter.begin());
+	} else {
+		if (origCov->rows != int(origCov->rownames.size()))
+			mxThrow("'%s' must have dimnames", origCov->name());
+		if (newCov->rows != int(newCov->rownames.size()))
+			mxThrow("'%s' must have dimnames", newCov->name());
+
+		for (int r1=0; r1 < int(newCov->rownames.size()); ++r1) {
+			bool found = false;
+			for (int r2=0; r2 < int(origCov->rownames.size()); ++r2) {
+				if (strEQ(newCov->rownames[r1], origCov->rownames[r2])) {
+					if (filter[r2]) {
+						omxRaiseErrorf("Cannot filter row '%s' in '%s' more than once",
+													 newCov->rownames[r1], origCov->name());
+					}
+					filter[r2] = true;
+					found = true;
+					break;
 				}
-				filter[r2] = true;
-				found = true;
-				break;
 			}
+			if (!found) omxRaiseErrorf("Cannot find row '%s' in '%s'",
+																 newCov->rownames[r1], origCov->name());
 		}
-		if (!found) omxRaiseErrorf("Cannot find row '%s' in '%s'",
-				     newCov->rownames[r1], origCov->name());
 	}
 }
 
@@ -2730,7 +2742,6 @@ static void pearsonSelCov(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 	omxMatrix *origCov = matList[0];
 	omxMatrix *newCov = matList[1];
 	EigenMatrixAdaptor EorigCov(origCov);
-	EigenMatrixAdaptor EnewCov(newCov);
 
 	omxResizeMatrix(result, origCov->rows, origCov->cols);
 	EigenMatrixAdaptor Eresult(result);
@@ -2740,9 +2751,22 @@ static void pearsonSelCov(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 	buildFilterVec(origCov, newCov, filter);
 	if (isErrorRaised()) return;
 
-	Eigen::MatrixXd v11(EnewCov.rows(), EnewCov.cols());
-	Eigen::MatrixXd v12(EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
-	Eigen::MatrixXd v22(EorigCov.rows() - EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
+	EigenMatrixAdaptor EnewCov(newCov);
+	Eigen::MatrixXd nc;
+	if (newCov->sameSize(origCov)) {
+		subsetCovariance(EnewCov, [&filter](int x)->bool{ return filter[x]; },
+										 std::accumulate(filter.begin(), filter.end(), 0), nc);
+	} else {
+		nc = EnewCov;
+	}
+	if (nc.rows() == 0) {
+		Eresult.derived() = EorigCov;
+		return;
+	}
+
+	Eigen::MatrixXd v11(nc.rows(), nc.cols());
+	Eigen::MatrixXd v12(nc.rows(), EorigCov.cols() - nc.cols());
+	Eigen::MatrixXd v22(EorigCov.rows() - nc.rows(), EorigCov.cols() - nc.cols());
 	partitionCovariance(EorigCov, [&](int xx){ return filter[xx]; },
 			    v11, v12, v22);
 
@@ -2752,11 +2776,11 @@ static void pearsonSelCov(FitContext *fc, omxMatrix** matList, int numArgs, omxM
 		return;
 	}
 	iv11 = iv11.selfadjointView<Eigen::Lower>();
-	Eigen::MatrixXd n12 = EnewCov * iv11 * v12;
-	Eigen::MatrixXd n22 = v22 - v12.transpose() * (iv11 - iv11 * EnewCov * iv11) * v12;
+	Eigen::MatrixXd n12 = nc * iv11 * v12;
+	Eigen::MatrixXd n22 = v22 - v12.transpose() * (iv11 - iv11 * nc * iv11) * v12;
 	
 	partitionCovarianceSet(Eresult, [&](int xx){ return filter[xx]; },
-			       EnewCov, n12, n22);
+			       nc, n12, n22);
 }
 
 static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omxMatrix* result)
@@ -2770,7 +2794,6 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 			 origMean->name(), origMean->rows, origCov->name(), origCov->rows);
 	}
 	EigenMatrixAdaptor EorigCov(origCov);
-	EigenMatrixAdaptor EnewCov(newCov);
 	EigenVectorAdaptor EorigMean(origMean);
 
 	omxResizeMatrix(result, EorigMean.size(), 1);
@@ -2781,9 +2804,22 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	buildFilterVec(origCov, newCov, filter);
 	if (isErrorRaised()) return;
 
-	Eigen::MatrixXd v11(EnewCov.rows(), EnewCov.cols());
-	Eigen::MatrixXd v12(EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
-	Eigen::MatrixXd v22(EorigCov.rows() - EnewCov.rows(), EorigCov.cols() - EnewCov.cols());
+	EigenMatrixAdaptor EnewCov(newCov);
+	Eigen::MatrixXd nc;
+	if (newCov->sameSize(origCov)) {
+		subsetCovariance(EnewCov, [&filter](int x)->bool{ return filter[x]; },
+										 std::accumulate(filter.begin(), filter.end(), 0), nc);
+	} else {
+		nc = EnewCov;
+	}
+	if (nc.rows() == 0) {
+		Eresult.derived() = EorigMean;
+		return;
+	}
+
+	Eigen::MatrixXd v11(nc.rows(), nc.cols());
+	Eigen::MatrixXd v12(nc.rows(), EorigCov.cols() - nc.cols());
+	Eigen::MatrixXd v22(EorigCov.rows() - nc.rows(), EorigCov.cols() - nc.cols());
 	partitionCovariance(EorigCov, [&](int xx){ return filter[xx]; },
 			    v11, v12, v22);
 
@@ -2796,7 +2832,7 @@ static void pearsonSelMean(FitContext *fc, omxMatrix** matList, int numArgs, omx
 	
 	Eigen::VectorXd obsMean;
 	subsetVector(EorigMean, [&](int xx){ return filter[xx]; },
-		     EnewCov.rows(), obsMean);
+		     nc.rows(), obsMean);
 
 	Eigen::VectorXd adj = v12.transpose() * iv11 * obsMean;
 	for (int v1=0, a1=0; v1 < EorigMean.rows(); ++v1) {
