@@ -1451,7 +1451,7 @@ void CSOLNP::subnp(Eigen::MatrixBase<T2>& pars, Eigen::MatrixBase<T1>& yy_e, Eig
 	hessv_e = hessv_e * vscale_e(0);
 	
 	if (verbose >= 1 && 1e-300 > tol) {
-		mxLog("m3 solnp mxThrow message being reported.");
+		mxLog("m3 solnp stop message being reported.");
 	}
 	
 	resP = p_e;
