@@ -31,6 +31,7 @@
 #define _OMXMATRIX_H_
 
 #include <R_ext/Arith.h>
+#include <memory>
 
 #include "omxDefines.h"
 #include <Eigen/Core>
@@ -71,6 +72,7 @@ class omxMatrix {
 								joinModel(0), shape(0), allocationLock(false),
 								freeRownames(false), freeColnames(false)
 		{};
+	struct dtor;
 	void setDependsOnParameters() { dependsOnParametersCache = true; };
 	void setDependsOnDefinitionVariables() { dependsOnDefVarCache = true; };
 	bool dependsOnParameters() const { return dependsOnParametersCache; };
@@ -118,6 +120,9 @@ class omxMatrix {
 	bool freeRownames, freeColnames;
 	std::vector<const char *> rownames;
 	std::vector<const char *> colnames;
+	bool hasDimnames() const { return rownames.size() && colnames.size(); }
+	bool sameSize(omxMatrix *other) const { return other->rows == rows && other->cols == cols; }
+	bool sameDimnames(omxMatrix *other) const;
 	int lookupColumnByName(const char *target);
 
 	friend void omxCopyMatrix(omxMatrix *dest, omxMatrix *src);  // turn into method later TODO
@@ -189,6 +194,12 @@ omxMatrix *omxCreateCopyOfMatrix(omxMatrix *orig, omxState *os);
 
 	void omxFreeMatrix(omxMatrix* om);						// Ditto, traversing argument trees
 
+struct omxMatrix::dtor {
+	void operator()(omxMatrix *om) { omxFreeMatrix(om); }
+};
+
+typedef std::unique_ptr< omxMatrix, omxMatrix::dtor > omxMatrixPtr;
+
 /* Matrix Creation Functions */
 omxMatrix* omxNewMatrixFromRPrimitive0(SEXP rObject, omxState* state, 
        unsigned short hasMatrixNumber, int matrixNumber);
@@ -462,8 +473,6 @@ void omxShallowInverse(int numIters, omxMatrix* A, omxMatrix* Z, omxMatrix* Ax,
 
 double omxMaxAbsDiff(omxMatrix *m1, omxMatrix *m2);
 
-bool thresholdsIncreasing(omxMatrix* om, int column, int count, FitContext *fc);
-
 void omxMatrixHorizCat(omxMatrix** matList, int numArgs, omxMatrix* result);
 
 void omxMatrixVertCat(omxMatrix** matList, int numArgs, omxMatrix* result);
