@@ -113,10 +113,13 @@ void omxRAMExpectation::compute(FitContext *fc, const char *what, const char *ho
 	}
 
 	oro->CalculateRAMCovarianceAndMeans(fc);
+
+	super::compute(fc, what, how);
 }
 
 void omxRAMExpectation::invalidateCache()
 {
+  super::invalidateCache();
 	if (rram) {
 		delete rram;
 		rram = 0;
@@ -125,17 +128,7 @@ void omxRAMExpectation::invalidateCache()
 
 omxRAMExpectation::~omxRAMExpectation()
 {
-	if(OMX_DEBUG) { mxLog("Destroying RAM Expectation."); }
-	
-	omxRAMExpectation* argStruct = this;
-
-	if (argStruct->rram) delete argStruct->rram;
-
-	omxFreeMatrix(argStruct->cov);
-
-	if(argStruct->means != NULL) {
-		omxFreeMatrix(argStruct->means);
-	}
+	if (rram) delete rram;
 }
 
 void omxRAMExpectation::populateAttr(SEXP robj)
@@ -167,13 +160,18 @@ void omxRAMExpectation::populateAttr(SEXP robj)
 	if (oro->rram) {
 		rram->exportInternalState(dbg);
 	} else {
-		oro->CalculateRAMCovarianceAndMeans(0);
+    // This code should be shared with all normal expectations TODO
+    compute(0, 0, 0);
 		EigenMatrixAdaptor Ecov(oro->cov);
 		out.add("covariance", Rcpp::wrap(Ecov));
 		if (oro->means) {
 			EigenVectorAdaptor Emean(oro->means);
 			out.add("mean", Rcpp::wrap(Emean));
 		}
+		if (hasProductNodes) {
+			dbg.add("polyRep", Rcpp::wrap(pcalc.getPolyRep()));
+		}
+    populateNormalAttr(robj, out);
 	}
 
 	Rf_setAttrib(robj, Rf_install("output"), out.asR());
@@ -202,16 +200,16 @@ void omxRAMExpectation::CalculateRAMCovarianceAndMeans(FitContext *fc)
 	if (F->rows == 0) return;
 
 	if (slope) omxRecompute(slope, fc);
-	    
+
 	if(cov == NULL && means == NULL) {
 		return; // We're not populating anything, so why bother running the calculation?
 	}
-	
+
 	EigenMatrixAdaptor Ecov(cov);
 	pcalc.cov(fc, Ecov);
 
 	if(OMX_DEBUG_ALGEBRA) {omxPrintMatrix(cov, "....RAM: Model-implied Covariance Matrix:");}
-	
+
 	if (M) {
 		EigenVectorAdaptor Emean(means);
 		pcalc.mean(fc, Emean);
@@ -242,7 +240,7 @@ static void recordNonzeroCoeff(omxMatrix *m, std::vector<coeffLoc> &vec, bool lo
 }
 
 omxRAMExpectation::omxRAMExpectation(omxState *st, int num)
-	: super(st, num), slope(0)
+	: super(st, num), studiedF(false), numExoPred(0), slope(0), rram(0)
 {
 	if (st->isClone()) {
 		omxRAMExpectation *ram = (omxRAMExpectation *)st->getParent(this);
@@ -255,18 +253,20 @@ omxRAMExpectation::omxRAMExpectation(omxState *st, int num)
 	}
 }
 
-void omxRAMExpectation::init() {
-	if(OMX_DEBUG) { mxLog("Initializing RAM expectation."); }
-	
+void omxRAMExpectation::init()
+{
+	loadDataColFromR();
+	loadThresholdFromR();
+
 	int l, k;
 
 	omxRAMExpectation *RAMexp = this;
 	RAMexp->rram = 0;
-	
+
 	auto oo=this;
 
 	oo->canDuplicate = true;
-	
+
 	ProtectedSEXP Rverbose(R_do_slot(rObj, Rf_install("verbose")));
 	RAMexp->verbose = Rf_asInteger(Rverbose) + OMX_DEBUG;
 
@@ -331,8 +331,9 @@ void omxRAMExpectation::init() {
 			if (!fex) mxThrow("%s: level transition matrix '%s' does not reference the upper level model",
 					   oo->name, bmat->name());
 			omxCompleteExpectation(fex);
-			if (!strEQ(fex->expType, "MxExpectationRAM")) {
-				mxThrow("%s: only MxExpectationRAM can be joined with MxExpectationRAM", oo->name);
+			if (!strEQ(fex->name, "MxExpectationRAM")) {
+				mxThrow("%s: only MxExpectationRAM can be joined with MxExpectationRAM (not %s)",
+								oo->name, fex->name);
 			}
 			omxDataKeysCompatible(fex->data, oo->data, foreignKey);
 			if (!omxDataColumnIsKey(oo->data, foreignKey)) {
@@ -346,7 +347,7 @@ void omxRAMExpectation::init() {
 				mxLog("%s: join col %d against %s using between matrix %s",
 				      oo->name, foreignKey, fex->name, bmat->name());
 			}
-				
+
 			RAMexp->between.push_back(bmat);
 		}
 	}
@@ -363,33 +364,38 @@ void omxRAMExpectation::init() {
 	isProductNode.assign(A->cols, false);
 	if (R_has_slot(rObj, Rf_install("isProductNode"))) {
 		ProtectedSEXP RprodNode(R_do_slot(rObj, Rf_install("isProductNode")));
-		if (Rf_length(RprodNode)) {
-			if (Rf_length(RprodNode) != A->cols) {
-				mxThrow("isProductNode must be same dimension as A matrix");
-			}
-			for (int px = 0; px < A->cols; ++px) {
-				if (INTEGER(RprodNode)[px]) {
-					isProductNode[px] = true;
-					hasProductNodes = true;
-				}
-			}
-		}
+    // PPML can transform a model such that RprodNode length does
+    // not match A->cols
+    if (Rf_length(RprodNode) == A->cols) {
+      for (int px = 0; px < A->cols; ++px) {
+        if (INTEGER(RprodNode)[px]) {
+          isProductNode[px] = true;
+          hasProductNodes = true;
+        }
+      }
+    }
 	}
 
 	if(OMX_DEBUG) { mxLog("Generating internals for computation."); }
 
-	RAMexp->cov = 		omxInitMatrix(l, l, TRUE, currentState);
+	cov = omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedCovariance", l, l);
+	if (!cov->hasMatrixNumber) covOwner = omxMatrixPtr(cov);
+	else connectMatrixToExpectation(cov, this, "covariance");
 
-	if(RAMexp->M != NULL) {
-		RAMexp->means = 	omxInitMatrix(1, l, TRUE, currentState);
+	if (M) {
+		means =	omxNewMatrixFromSlotOrAnon(rObj, currentState, "expectedMean", 1, l);
+		if (!means->hasMatrixNumber) meanOwner = omxMatrixPtr(means);
+		else connectMatrixToExpectation(means, this, "mean");
 	} else {
-	    RAMexp->means  = 	NULL;
-    }
+		RAMexp->means  = 	NULL;
+	}
 
 	RAMexp->studyF();
 	//mxPrintMat("RAM corrected dc", oo->getDataColumns());
 
 	if (!currentState->isClone()) {
+		if (hasProductNodes) rampartCycleLimit = 0;
+
 		Eigen::VectorXd estSave;
 		currentState->setFakeParam(estSave);
 		loadFakeDefVars();
@@ -410,7 +416,14 @@ void omxRAMExpectation::init() {
 		pcalc.setAlgo(0, hasProductNodes, useSparse);
 
 		currentState->restoreParam(estSave);
-	}
+	} else {
+    auto pex = (omxRAMExpectation*) currentState->getParent(this);
+    if (pex->slope) {
+      numExoPred = pex->numExoPred;
+      exoDataColIndex = pex->exoDataColIndex;
+      addSlopeMatrix();
+    }
+  }
 }
 
 void omxRAMExpectation::studyExoPred()
@@ -425,8 +438,7 @@ void omxRAMExpectation::studyExoPred()
 	EigenMatrixAdaptor eS(S);
 	hasVariance = eS.diagonal().array().abs().matrix();
 
-	int found = 0;
-	std::vector<int> exoDataCol(S->rows, -1);
+  exoDataColIndex.resize(S->rows, -1);
 	int mNum = ~M->matrixNumber;
 	for (int k=0; k < int(data->defVars.size()); ++k) {
 		omxDefinitionVar &dv = data->defVars[k];
@@ -441,8 +453,8 @@ void omxRAMExpectation::studyExoPred()
 			if (!toManifest && !latentName) continue;
 			if (latentName) mxThrow("%s: latent exogenous variables are not supported (%s -> %s)", name,
 						 S->colnames[dv.col], latentName);
-			exoDataCol[dv.col] = dv.column;
-			found += 1;
+			exoDataColIndex[dv.col] = dv.column;
+			numExoPred += 1;
 			dv.loadData(currentState, 0.);
 			if (OMX_DEBUG + verbose >= 1) {
 				mxLog("%s: set defvar '%s' for latent '%s' to exogenous mode",
@@ -454,21 +466,26 @@ void omxRAMExpectation::studyExoPred()
 
 	currentState->restoreParam(estSave);
 
-	if (!found) return;
+  addSlopeMatrix();
+}
 
-	slope = omxInitMatrix(F->rows, found, currentState);
+void omxRAMExpectation::addSlopeMatrix()
+{
+	if (!numExoPred) return;
+
+	slope = omxInitMatrix(F->rows, numExoPred, currentState);
 	EigenMatrixAdaptor eSl(slope);
 	eSl.setZero();
 
 	for (int cx=0, ex=0; cx < S->rows; ++cx) {
-		if (exoDataCol[cx] == -1) continue;
-		auto &rc = data->rawCols[ exoDataCol[cx] ];
+		if (exoDataColIndex[cx] == -1) continue;
+		auto &rc = data->rawCol( exoDataColIndex[cx] );
 		if (rc.type != COLUMNDATA_NUMERIC) {
 			omxRaiseErrorf("%s: exogenous predictor '%s' must be type numeric (not '%s')",
 				       name, rc.name, rc.typeName());
 			continue;
 		}
-		exoDataColumns.push_back(exoDataCol[cx]);
+		exoDataColumns.push_back(exoDataColIndex[cx]);
 		for (int rx=0, dx=0; rx < S->rows; ++rx) {
 			if (!latentFilter[rx]) continue;
 			slope->addPopulate(A, rx, cx, dx, ex);
@@ -476,13 +493,18 @@ void omxRAMExpectation::studyExoPred()
 		}
 		ex += 1;
 	}
+}
+
+void omxRAMExpectation::connectToData()
+{
+  super::connectToData();
 
 	exoPredMean.resize(exoDataColumns.size());
 	for (int cx=0; cx < int(exoDataColumns.size()); ++cx) {
-               auto &e1 = data->rawCols[ exoDataColumns[cx] ];
-               Eigen::Map< Eigen::VectorXd > vec(e1.ptr.realData, data->numRawRows());
-               exoPredMean[cx] = vec.mean();
-       }
+		auto &e1 = data->rawCol( exoDataColumns[cx] );
+		Eigen::Map< Eigen::VectorXd > vec(e1.d(), data->numRawRows());
+		exoPredMean[cx] = vec.mean();
+	}
 }
 
 void omxRAMExpectation::studyF()
@@ -508,12 +530,18 @@ void omxRAMExpectation::studyF()
 			dataCols[dx] = newDest;
 			if (origThresholdInfo.size()) {
 				omxThresholdColumn adj = origThresholdInfo[dest];
-				adj.dColumn = dx;
+				adj.dataColumn = newDest;
 				thresholds.push_back(adj);
 			}
 			dx += 1;
 		}
 	}
+  // ensure we don't try to use the wrong stuff
+  origDataColumnNames.clear();
+  origThresholdInfo.clear();
+  studiedF = true;
+  //mxLog("studyF: dataColumns permuted to (%d):", int(dataColNames.size()));
+  //for (auto &dc : dataColNames) mxLog("%s", dc);
 }
 
 omxMatrix* omxRAMExpectation::getComponent(const char* component)
@@ -533,7 +561,7 @@ omxMatrix* omxRAMExpectation::getComponent(const char* component)
 	} else if(strEQ("pvec", component)) {
 		// Once implemented, change compute function and return pvec
 	}
-	
+
 	return retval;
 }
 
@@ -566,7 +594,7 @@ void omxRAMExpectation::analyzeDefVars(FitContext *fc)
 		dvInfluenceMean.assign(numDefVars, M? true : false);
 		if (verbose >= 1) mxLog("%s: defvar effect on mean unknown", name);
 	}
-		
+
 	hasVariance.resize(S->rows);
 	if (S->isSimple()) {
 		omxRecompute(S, fc);
@@ -756,7 +784,7 @@ namespace RelationalRAMExpectation {
 
 	unsigned independentGroup::ApcIO::getVersion(FitContext *fc)
 	{
-		unsigned v = 0;
+		unsigned v = useRampart * 100000;
 		for (int ax=0; ax < clumpSize; ++ax) {
 			addr &a1 = par.st.layout[ par.gMap[ax] ];
 			omxExpectation *expectation = a1.getModel(fc);
@@ -796,8 +824,11 @@ namespace RelationalRAMExpectation {
 				for (auto &v : *ram->Acoeff) mat.coeffRef(pl.modelStart + v.c, pl.modelStart + v.r) = -dat[v.off];
 			}
 
-			const double scale = a1.rampartScale;
-			if (scale == 0.0) continue;
+			double scale = 1.0;
+			if (useRampart) {
+				scale = a1.rampartScale;
+				if (scale == 0.0) continue;
+			}
 
 			omxData *data = expectation->data;
 			for (size_t jx=0; jx < ram->between.size(); ++jx) {
@@ -1224,7 +1255,7 @@ namespace RelationalRAMExpectation {
 						      hasMean, hasVar);
 					}
 				}
-				
+
 				if (hasMean || hasVar) {
 					for (auto it : fromDv) {
 						omxRAMExpectation *dvHome =
@@ -1490,7 +1521,9 @@ namespace RelationalRAMExpectation {
 		arrayIndex = ig->arrayIndex;
 		obsNameVec = 0;
 		varNameVec = 0;
+		skipMean = ig->skipMean;
 		expectedVec.resize(ig->expectedVec.size());
+		if (skipMean) expectedVec.setZero();
 		fullMean.resize(ig->fullMean.size());
 		fullMean.setZero();
 		clumpVars = ig->clumpVars;
@@ -1500,6 +1533,7 @@ namespace RelationalRAMExpectation {
 
 	void independentGroup::prep(FitContext *fc)
 	{
+		skipMean = NA_INTEGER;
 		int totalObserved = 0;
 		int maxSize = 0;
 		if (placements.size()) {
@@ -1515,9 +1549,7 @@ namespace RelationalRAMExpectation {
 		}
 		latentFilter.assign(maxSize, false); // will have totalObserved true entries
 		isProductNode.assign(maxSize, false);
-		obsNameVec = Rf_protect(Rf_allocVector(STRSXP, totalObserved));
 		varNameVec = Rf_protect(Rf_allocVector(STRSXP, maxSize));
-		expectedVec.resize(totalObserved);
 		dataVec.resize(totalObserved);
 		dataColumn.resize(totalObserved);
 		dataColumn.setConstant(-1);
@@ -1535,6 +1567,9 @@ namespace RelationalRAMExpectation {
 			clumpVars = end.modelStart + a1.numVars();
 			clumpObs = end.obsStart + a1.numObs();
 		}
+		int tmpObs = st.hasProductNodes? clumpObs : totalObserved;
+		expectedVec.resize(tmpObs);
+		obsNameVec = Rf_protect(Rf_allocVector(STRSXP, tmpObs));
 
 		int dx=0;
 		for (size_t ax=0; ax < placements.size(); ++ax) {
@@ -1576,7 +1611,7 @@ namespace RelationalRAMExpectation {
 					latentFilter[ pl.modelStart + vx ] = true;
 					std::string dname =
 						modelName + omxDataColumnName(data, dc[col]);
-					SET_STRING_ELT(obsNameVec, dx, Rf_mkChar(dname.c_str()));
+					if (dx < tmpObs) SET_STRING_ELT(obsNameVec, dx, Rf_mkChar(dname.c_str()));
 					dataVec[ dx ] = val;
 					if (a1.getExpNum() == st.homeEx->expNum) dataColumn[ dx ] = col;
 					dx += 1;
@@ -1854,7 +1889,7 @@ namespace RelationalRAMExpectation {
 		}
 
 		int maxSize = 0;
-		int homeDataRows = homeEx->data->rows;
+		int homeDataRows = homeEx->data->nrows();
 		for (int row=0; row < homeDataRows; ++row) {
 			flattenOneRow(homeEx, row, maxSize);
 			if (isErrorRaised()) return;
@@ -1960,7 +1995,7 @@ namespace RelationalRAMExpectation {
 		for (std::vector<independentGroup*>::iterator it = group.begin() ; it != group.end(); ++it) {
 			delete *it;
 		}
-  
+
 		omxFreeMatrix(smallCol);
 	}
 
@@ -1986,12 +2021,17 @@ namespace RelationalRAMExpectation {
 				}
 			}
 		}
+		if (st.hasProductNodes) {
+			skipMean = st.layoutSetup[gMap[0]].skipMean;
+			if (skipMean) expectedVec.setZero();
+		}
 	}
 
 	void independentGroup::computeCov(FitContext *fc)
 	{
 		if (0 == getParent().dataVec.size()) return;
 
+		((ApcIO&)*pcalc.aio).useRampart = true;
 		pcalc.cov(fc, fullCov);
 	}
 
@@ -1999,6 +2039,9 @@ namespace RelationalRAMExpectation {
 	{
 		if (0 == getParent().dataVec.size()) return;
 
+		if (skipMean) return;
+
+		((ApcIO&)*pcalc.aio).useRampart = false;
 		pcalc.mean(fc, expectedVec);
 	}
 
@@ -2063,21 +2106,22 @@ namespace RelationalRAMExpectation {
 			auto &dc = ex1->getDataColumns();
 			if (dc.size() == 0) continue;
 			omxData *data = ex1->data;
+			int rows = data->nrows();
 
 			SEXP df;
 			Rf_protect(df = Rf_allocVector(VECSXP, dc.size()));
 			SEXP colnames = Rf_allocVector(STRSXP, dc.size());
 			Rf_setAttrib(df, R_NamesSymbol, colnames);
 			for (int col=0; col < int(dc.size()); ++col) {
-				SEXP colData = Rf_allocVector(REALSXP, data->rows);
+				SEXP colData = Rf_allocVector(REALSXP, rows);
 				SET_VECTOR_ELT(df, col, colData);
 				double *colPtr = REAL(colData);
-				for (int rx=0; rx < data->rows; ++rx) {
+				for (int rx=0; rx < rows; ++rx) {
 					colPtr[rx] = NA_REAL;
 				}
 				SET_STRING_ELT(colnames, col, Rf_mkChar(omxDataColumnName(data, dc[col])));
 			}
-			markAsDataFrame(df, data->rows);
+			markAsDataFrame(df, rows);
 
 			DataMap[ex1] = df;
 			out.add(data->name, df);
@@ -2131,6 +2175,18 @@ namespace RelationalRAMExpectation {
 		} else {
 			computeMeanByModel(fc);
 		}
+
+		state &pst = getParent();
+		const int layoutSize = pst.layout.size();
+		if (pst.getOptimizeMean() >= 1) {
+			for (int ax=0; ax < layoutSize; ax += pst.layout[ax].nextMean) {
+				addr &a1 = pst.layout[ax];
+				int a1Start = a1.ig->placements[a1.igIndex].obsStart;
+				independentGroup &tig1 = *group[a1.ig->arrayIndex];
+				tig1.expectedVec.segment(a1Start, a1.numObs())
+					*= a1.quickRotationFactor;
+			}
+		}
 	}
 
 	void state::computeMeanByGroup(FitContext *fc)
@@ -2214,16 +2270,6 @@ namespace RelationalRAMExpectation {
 			}
 		}
 
-		if (pst.getOptimizeMean() >= 1) {
-			for (int ax=0; ax < layoutSize; ax += pst.layout[ax].nextMean) {
-				addr &a1 = pst.layout[ax];
-				int a1Start = a1.ig->placements[a1.igIndex].obsStart;
-				independentGroup &tig1 = *group[a1.ig->arrayIndex];
-				tig1.expectedVec.segment(a1Start, a1.numObs())
-					*= a1.quickRotationFactor;
-			}
-		}
-
 		pst.applyRotationPlan(UnitAccessor<true>(this));
 	}
 
@@ -2237,6 +2283,10 @@ namespace RelationalRAMExpectation {
 		dbg.add("clumpSize", Rf_ScalarInteger(clumpSize));
 		dbg.add("clumpObs", Rf_ScalarInteger(clumpObs));
 		dbg.add("numLooseClumps", Rf_ScalarInteger(numLooseClumps()));
+		dbg.add("skipMean", Rf_ScalarInteger(skipMean));
+		if (st.hasProductNodes) {
+			dbg.add("polyRep", Rcpp::wrap(pcalc.getPolyRep()));
+		}
 
 		if (clumpObs < 500) {
 			// Can crash R because vectors are too long.
@@ -2270,7 +2320,7 @@ namespace RelationalRAMExpectation {
 			dbg.add("latentFilter", Rcpp::wrap(latentFilter));
 			SEXP dv = Rcpp::wrap(dataVec);
 			Rf_protect(dv);
-			Rf_setAttrib(dv, R_NamesSymbol, obsNameVec);
+			//Rf_setAttrib(dv, R_NamesSymbol, obsNameVec);
 			dbg.add("dataVec", dv);
 		} else {
 			Rf_warning("%s: group %d too large to transfer to back to R",
@@ -2322,8 +2372,9 @@ namespace RelationalRAMExpectation {
 		dbg.add("rampartUsage", Rcpp::wrap(rampartUsage));
 		dbg.add("rotationCount", Rcpp::wrap(rotationCount));
 		dbg.add("numGroups", Rcpp::wrap(int(group.size())));
+		dbg.add("hasProductNodes", Rcpp::wrap(hasProductNodes));
 
-		SEXP modelName, row, numJoins, numKids, parent1, fk1, rscale,
+		SEXP modelName, row, numJoins, numKids, parent1, fk1, rscale, qrf,
 			hmean, skipMean, rset, ugroup;
 		Rf_protect(modelName = Rf_allocVector(STRSXP, layout.size()));
 		Rf_protect(row = Rf_allocVector(INTSXP, layout.size()));
@@ -2332,6 +2383,7 @@ namespace RelationalRAMExpectation {
 		Rf_protect(parent1 = Rf_allocVector(INTSXP, layout.size()));
 		Rf_protect(fk1 = Rf_allocVector(INTSXP, layout.size()));
 		Rf_protect(rscale = Rf_allocVector(REALSXP, layout.size()));
+		Rf_protect(qrf = Rf_allocVector(REALSXP, layout.size()));
 		Rf_protect(hmean = Rf_allocVector(INTSXP, layout.size()));
 		Rf_protect(skipMean = Rf_allocVector(INTSXP, layout.size()));
 		Rf_protect(rset = Rf_allocVector(INTSXP, layout.size()));
@@ -2344,6 +2396,7 @@ namespace RelationalRAMExpectation {
 			INTEGER(parent1)[mx] = plusOne(layoutSetup[mx].parent1);
 			INTEGER(fk1)[mx] = layoutSetup[mx].fk1;
 			REAL(rscale)[mx] = layout[mx].rampartScale;
+			REAL(qrf)[mx] = layout[mx].quickRotationFactor;
 			INTEGER(hmean)[mx] = layoutSetup[mx].heterogenousMean;
 			INTEGER(skipMean)[mx] = layoutSetup[mx].skipMean;
 			INTEGER(rset)[mx] = layoutSetup[mx].rset;
@@ -2356,6 +2409,7 @@ namespace RelationalRAMExpectation {
 							  Rcpp::Named("parent1")=parent1,
 							  Rcpp::Named("fk1")=fk1,
 							  Rcpp::Named("rampartScale")=rscale,
+							  Rcpp::Named("qrf")=qrf,
 							  Rcpp::Named("hmean")=hmean,
 							  Rcpp::Named("skip")=skipMean,
 							  Rcpp::Named("rset")=rset,
