@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -22,9 +22,8 @@ context("MatrixErrorDetection")
 A <- mxMatrix('Full', 1, 1, labels = 'data.foo', free = TRUE, name = 'A')
 model <- mxModel('model', A)
 omxCheckError(mxRun(model), 
-	paste("The definition variable 'data.foo'",
-		"has been assigned to a",
-		"free parameter in matrix 'A'"))
+	paste("matrix 'model.A' free=TRUE but label looks",
+	      "like a definition variable:"))
 
 A <- mxMatrix('Full', 1, 1, labels = 'foo[1,2]', free = TRUE, name = 'A')
 model <- mxModel('model', A)
